@@ -3,6 +3,7 @@
 Contains interface (MultiDomainBasicAuth) and associated glue code for
 providing credentials in the context of network requests.
 """
+
 import logging
 import os
 import shutil
@@ -47,12 +48,12 @@ class KeyRingBaseProvider(ABC):
     has_keyring: bool
 
     @abstractmethod
-    def get_auth_info(self, url: str, username: Optional[str]) -> Optional[AuthInfo]:
-        ...
+    def get_auth_info(
+        self, url: str, username: Optional[str]
+    ) -> Optional[AuthInfo]: ...
 
     @abstractmethod
-    def save_auth_info(self, url: str, username: str, password: str) -> None:
-        ...
+    def save_auth_info(self, url: str, username: str, password: str) -> None: ...
 
 
 class KeyRingNullProvider(KeyRingBaseProvider):
@@ -151,7 +152,7 @@ class KeyRingCliProvider(KeyRingBaseProvider):
         env["PYTHONIOENCODING"] = "utf-8"
         subprocess.run(
             [self.keyring, "set", service_name, username],
-            input=f"{password}{os.linesep}".encode("utf-8"),
+            input=f"{password}{os.linesep}".encode(),
             env=env,
             check=True,
         )
