@@ -1,12 +1,9 @@
 import importlib.util
 import json
+import logging
 import os
 import sys
-
-try:
-    from functools import cached_property
-except ImportError:
-    cached_property = property
+from functools import cached_property
 
 
 def _ensure_modules():
@@ -82,7 +79,11 @@ def handle_parsed_args(parsed):
         with open(parsed.constraints_file) as constraints:
             file_constraints = constraints.read().strip().split("\n")
         os.unlink(parsed.constraints_file)
-        parsed.packages += sorted(file_constraints)
+        packages = {}
+        for line in file_constraints:
+            dep_name, pip_line = line.split(",", 1)
+            packages[dep_name] = pip_line
+        parsed.packages = packages
     return parsed
 
 
@@ -148,42 +149,59 @@ class Entry:
         return entry_dict
 
     @classmethod
-    def parse_pyparsing_exprs(cls, expr_iterable):
-        from pipenv.patched.pip._vendor.pyparsing import Literal, MatchFirst
-
+    def parse_expressions(cls, expr_iterable):
         keys = []
         expr_list = []
         expr = expr_iterable.copy()
-        if isinstance(expr, Literal) or (expr.__class__.__name__ == Literal.__name__):
-            keys.append(expr.match)
-        elif isinstance(expr, MatchFirst) or (
-            expr.__class__.__name__ == MatchFirst.__name__
-        ):
-            expr_list = expr.exprs
-        elif isinstance(expr, list):
+        if isinstance(expr, list):
             expr_list = expr
         if expr_list:
             for part in expr_list:
-                keys.extend(cls.parse_pyparsing_exprs(part))
+                keys.extend(cls.parse_expressions(part))
         return keys
 
     @classmethod
     def get_markers_from_dict(cls, entry_dict):
-        from pipenv.patched.pip._vendor.packaging import markers as packaging_markers
+        from pipenv.patched.pip._vendor.packaging._parser import (
+            parse_marker as packaging_parse_marker,
+        )
         from pipenv.utils.markers import normalize_marker_str
 
-        marker_keys = cls.parse_pyparsing_exprs(packaging_markers.VARIABLE)
         markers = set()
+
+        # If no markers are present, skip marker parsing
+        if not any(
+            k in entry_dict
+            for k in ["sys_platform", "python_version", "os_name", "platform_machine"]
+        ):
+            return None, entry_dict
+
+        # Otherwise, proceed to parse markers from entry_dict
+        marker_expression = packaging_parse_marker(entry_dict.get("markers", ""))
+
+        # Parse the marker expressions using the new packaging marker parser
+        marker_keys = cls.parse_expressions(marker_expression)
+
+        # Identify relevant marker keys present in entry_dict
         keys_in_dict = [k for k in marker_keys if k in entry_dict]
+
+        # Normalize and add the markers from entry_dict
         markers = {normalize_marker_str(f"{k} {entry_dict.pop(k)}") for k in keys_in_dict}
+
+        # Handle "markers" field if it exists in the dictionary
         if "markers" in entry_dict:
             markers.add(normalize_marker_str(entry_dict["markers"]))
+
+        # Remove None from the set if present
         if None in markers:
             markers.remove(None)
+
+        # If there are any markers left, join them with "and"
         if markers:
             entry_dict["markers"] = " and ".join(list(markers))
         else:
             markers = None
+
         return markers, entry_dict
 
     @property
@@ -286,7 +304,7 @@ class Entry:
 
     @property
     def is_in_pipfile(self):
-        return True if self.pipfile_name else False
+        return bool(self.pipfile_name)
 
     @property
     def pipfile_packages(self):
@@ -315,7 +333,7 @@ class Entry:
     def clean_specifier(specifier):
         from pipenv.patched.pip._vendor.packaging.specifiers import Specifier
 
-        if not any(specifier.startswith(k) for k in Specifier._operators.keys()):
+        if not any(specifier.startswith(k) for k in Specifier._operators):
             if specifier.strip().lower() in ["any", "<any>", "*"]:
                 return "*"
             specifier = f"=={specifier}"
@@ -327,14 +345,12 @@ class Entry:
     def strip_version(specifier):
         from pipenv.patched.pip._vendor.packaging.specifiers import Specifier
 
-        op = next(
-            iter(k for k in Specifier._operators.keys() if specifier.startswith(k)), None
-        )
+        op = next(iter(k for k in Specifier._operators if specifier.startswith(k)), None)
         if op:
             specifier = specifier[len(op) :]
         while op:
             op = next(
-                iter(k for k in Specifier._operators.keys() if specifier.startswith(k)),
+                iter(k for k in Specifier._operators if specifier.startswith(k)),
                 None,
             )
             if op:
@@ -580,12 +596,8 @@ def resolve_packages(
         else None
     )
 
-    if not isinstance(packages, set):
-        packages = set(packages)
-    if not isinstance(constraints, set):
-        constraints = set(constraints) if constraints else set()
     if constraints:
-        packages |= constraints
+        packages.update(constraints)
 
     def resolve(
         packages, pre, project, sources, clear, system, category, requirements_dir=None
@@ -656,6 +668,9 @@ def main(argv=None):
     os.environ["PYTHONIOENCODING"] = "utf-8"
     os.environ["PYTHONUNBUFFERED"] = "1"
     parsed = handle_parsed_args(parsed)
+    if not parsed.verbose:
+        print(parsed.verbose)
+        logging.getLogger("pipenv").setLevel(logging.WARN)
     _main(
         parsed.pre,
         parsed.clear,
